/*****************************************************************************/
/*                                                                           */
/*  HyperPanel VMOS                     File "include/drv_gpio.h"            */
/*  Constants and definitions for GPIO.                                      */
/*                                                                           */
/*  Author : JYA                        Last modification : 02/11/2016       */
/*                                                                           */
/*  (c) Copyright HyperPanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations =======================================================


         1  Defines
         2  Structures and typedefs
            2.1  High level GPIO driver definitions - Gpio_def
            2.2  Low level GPIO driver interface - Gpio_desc
            2.3  GPIO controller context - Gpioctl
            2.4  GPIO pin context - Gpiopin
         3  Public variable
         4  Prototypes of public procedures


*/


#ifndef DRV_GPIO_H                     /* If this file is not already        */
#define DRV_GPIO_H                     /* included, include it.              */


/*
    1    Defines
    ____________
*/

/*
   2.1  High level GPIO interface - Registers and tags
   ---------------------------------------------------
*/

#define TAG_INPUT 10                   /* For IND_REPORT events.             */


/* Number of registers for tags of type TAG_STR .............................*/

#define NBRNAME              2         /* 2 registers for controller name    */
#define NBRPINS             (2*256)    /* 2 registers per pin x 256 pins     */
#define NBRPNAME             2         /* 2 registers per pin                */
#define NBRFUNCS            (2* 32)    /* 2 registers per func x 32 funcs    */
#define NBRFUNC              2         /* 2 registers per function           */

/* Controller registers .....................................................*/

#define NRTYPE               0         /* string  TYPE      (  1 UINT )      */
#define NRNAME               1         /* string  NAME      (  2 UINTs)      */
#define NRNUM                3         /* integer NUM       (  1 UINT )      */
#define NRRATE               4         /* integer RATE      (  1 UINT )      */
#define NRMISS               5         /* integer MISS      (  1 UINT )      */
#define NRPINS               6         /* string  PINS      (512 UINTs)      */

/* Pin registers ............................................................*/

#define NRPNAME            518         /* string  PNAME     (  2 UINTs)      */
#define NRFUNCS            520         /* string  FUNCS                      */
#define NRFUNC             521         /* string  FUNC      (  1 UINT )      */
#define NRCFG              522         /* integer CFG       (  1 UINT )      */
#define NREDGE             523         /* integer EDGE      (  1 UINT )      */
#define NRCNTF             524         /* integer CNTF      (  1 UINT )      */
#define NRCNTR             525         /* integer CNTR      (  1 UINT )      */

/* Size of table of registers for controller and pin ........................*/

#define NBREGCTL           518         /* Controller registers (0 to 517)    */
#define NBREGPIN             8         /* Pin registers (518 to 525)         */


/*
   2.2  Low level GPIO interface - Mnemonics for function parameters
   -----------------------------------------------------------------
*/

/* GPIO static functions ....................................................*/

#define FNC_GPIO             0         /* Function GPIO (0). Mandatory.      */

/* GPIO output state ........................................................*/

#define GPIO_LOW             0         /* Low (-V or ground).                */
#define GPIO_HIGH            1         /* High (+V).                         */
#define GPIO_HIZ             2         /* High impedance (floating).         */
#define GPIO_OFF             3         /* No output.                         */

/* GPIO weak-pull configuration .............................................*/

#define GPIO_NOPULL          0         /* No pull-up or pull-down.           */
#define GPIO_PU              1         /* Weak pull-up.                      */
#define GPIO_PD              2         /* Weak pull-down.                    */



/*
    2    Structures and typedefs
    ____________________________
*/


/*
   2.1  High level GPIO driver definitions - Gpio_def
   --------------------------------------------------
*/


typedef struct                         /* GPIO pin init.                     */
  BEGIN                                /* ---------------------------------- */
    const                              /*                                    */
    CHAR* const     pname            ; /* GPIO pin name                      */
    INT             hidden           ; /* Hidden flag                        */
    const CHAR     *func             ; /* Function                           */
    INT             outcfg           ; /* Output configuration               */
    INT             pull             ; /* Pull configuration                 */
#if 0 // @TODO
    void          (*fall_cb)()       ; /* Falling edge callback              */
    void          (*rise_cb)()       ; /* Rising edge callback               */
#endif
  END               Gpio_pindef      ; /* ---------------------------------- */


typedef struct                         /* GPIO driver definitions.           */
  BEGIN                                /* ---------------------------------- */
    Gpio_pindef    *pindefs          ; /* Definitions for init. of pins.     */
  END               Gpio_def         ; /* ---------------------------------- */


/*
   2.2  Low level GPIO driver interface - Gpio_desc
   ------------------------------------------------
*/


typedef struct                         /* Adresses proceduresde bas niveau  */
  BEGIN                                /* ---------------------------------  */
    void            (*info       )() ; /* Recuperation des informations.     */
    void            (*get_cap    )() ; /* Lecture des capacites d'une pin.   */
    void            (*set_func   )() ; /* Choix d'une fonction.              */
    void            (*get_input  )() ; /* Lecture de l'etat d'une PIN.       */
    void            (*set_output )() ; /* Pilotage de l'etat de sortie.      */
    void            (*get_output )() ; /* Lecture de l'etat de sortie.       */
    void            (*set_pull   )() ; /* Configuration du "weak pull".      */
    void            (*get_pull   )() ; /* Lecture de l'etat du "weak pull".  */
    int             (*set_irq    )() ; /* Configuration des interruptions.   */
  END               Gpio_desc        ; /* ---------------------------------- */


/*
   2.3  GPIO controller context - Gpioctl
   --------------------------------------
*/


typedef struct                         /* GPIO controller context            */
  BEGIN                                /* ---------------------------------- */
    Req_ctx         rctx             ; /* Allocation context.                */
    UCHAR           fpin             ; /* First pin index.                   */
    Gpio_desc      *desc             ; /* Low-level driver interface.        */
    Gpio_def       *def              ; /* Low-level driver interface.        */
    CHAR           *name             ; /* Controller's name.                 */
    INT             nbfunc           ; /* Number of supported functions.     */
    CHAR          **tfunc            ; /* Table of supported functions.      */
    INT             nbpins           ; /* Number of pins drived by controller*/
    CHAR          **tpins            ; /* Table of pins names.               */
                                       /* ---------------------------------- */
/* @TODO What follows need reset at close.                                   */
    UINT            reg[NBREGCTL]    ; /* Missed events counter.             */
  END               Gpioctl          ; /* ---------------------------------- */
     

/*
   2.4  GPIO pin context - Gpiopin
   -------------------------------
*/


typedef struct                         /* GPIO pin context                   */
  BEGIN                                /* ---------------------------------- */
    Gpio_pindef    *def              ; /* Pin definition if any.             */
    UINT            iodsub           ; /* Subchannel IOD.                    */
    UCHAR           dev              ; /* Controller's device identifier.    */
    UCHAR           pin              ; /* Controller's pin identifier.       */
    USHORT          vl               ; /* Logical channel in AUT_GPIO.       */
    USHORT          reqemet          ; /* Allocator automaton.               */
    USHORT          reqevl           ; /* Allocator logical channel          */
    USHORT          reqnode          ; /* Allocator node.                    */
    INT             idto             ; /* Current timer id.                  */
                                       /* ---------------------------------- */
    UINT            reg[NBREGPIN]    ; /* Missed events counter.             */
    UCHAR           imsk             ; /* Interrupt mask.                    */
    UCHAR           sval             ; /* Saved input value (debounce hold). */
#if 0
    UCHAR           func             ; /* Current function index.            */
    UCHAR           out              ; /* Current output configuration.      */
    USHORT          keycode          ; /* Key code value.                    */
    UCHAR           deb              ; /* Debounce duration.                 */
    UCHAR           evts             ; /* Mask for events.                   */
    UCHAR           cnts             ; /* Mask for  mask.                    */
    UINT            cnt[2]           ; /* Edges counter (0:Falling,1:Rising) */
#endif
  END               Gpiopin          ; /* ---------------------------------- */


/*
    3    Public variables
    _____________________
*/


/*
    4    Prototypes of public procedures
    ____________________________________
*/


  void  sig_input           (INT,INT                                         );


#endif                                 /* End of DRV_GPIO_H.                 */
