/*****************************************************************************/
/*                                                                           */
/*  HyperPanel VMOS                     File "vmio/stm32/board_stm32m3.c"    */
/*  Nucleo L152RE board description (STM32).                                 */
/*                                                                           */
/*  Author : PJB                        Last modification : 09/09/2015       */
/*                                                                           */
/*  (c) Copyright HyperPanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/

 
/*  Some explanations ========================================================



    1    Purpose of this module
    ___________________________


         Ce module contient les donnees de configuration, les parametres des
drivers VMIO de la plateforme Nucleo L152RE. Il contient:

         - Les parametres des drivers systemes

         - Les parametres des drivers caracteres

         - Les parametres des drivers blocs

         - Les parametres des drivers volumes

         - Les parametres des protocoles

         - Les tables qui regroupent les adresses des donnees precedentes

         - Les quelques procedures C specifiques de ce board.


         Dans la classification des produits STM32L1xx, il existe une notion
de categorie. Le STM32L152RE qui peuple les carte Nucleo-64 (MB1136) est de
categorie 5.



    2    User manual
    ________________


    +    2.1  Utilisation de la declaration des boards
    +    2.2  Schema general



    2.1  Utilisation de la declaration des boards

         Une carte particuliere est decrite par une arborescence de structures
C, dont la structure "racine" est toujours une structure de type "Board_link".
Les structures "Board_link" fournies par le present modules se trouvent au
point "11.1 Boards".



    2.2  Schema general

         Le schema qui suit est une copie du schema que l'on trouve au point
"2.9.1  Generalites - Schema general" de "drv_itf1.c". Pour plus
d'explications, on se reportera donc a "drv_itf1.c".

#DIAGRAM ----------------------------------------------------------------------

INT            Board_link *                           Description des boards
num_board      boards[]
+--------+     +--------+
|        | -+  |        |
+--------+  |  +--------+
            |  |        |      Board_link
            |  +--------+      +------------+
            +->| ADBOARD| -+-> | board_name |-----+
               +--------+  |   | drv_link   |     |
               |        |  |   +------------+     |
               +--------+  |   Une structure      |
               |        |  |   Board_link         |
               +--------+  |                      |
               |        |  |                      |
               +--------+  |                      |
    Tableau d'adresses de  |                      |
    structures Board_link  |                      |
                           |                      |
   Board_link* +--------+  |                      |
        board  | ADBOARD| -+                      |
               +--------+                         |
                                                  |
                                                  |
   +----------------------------------------------+
===|===========================================================================
   |   Drv_link * []                                 Descriptions des drivers
   |   +-----------+                                 systemes
   +-> |           |     Sdrv_link          Sdrv_desc
       +-----------+     +-----------+      +----------+
       |           | --> | sdrv_desc | ---> | drv_name |
       +-----------+     | dev_link  | -+   | typdrv   |
       |           |     | nb_dev    |  |   | init     | --> proc C
       +-----------+     +-----------+  |   | end      | --> proc C
       |           |      Une structure |   +----------+
       +-----------+      Sdrv_link     |
       |           |                    |   Dev_link[]
       +-----------+                    |   +----------+      +-----+
       |           |                    +-> | dev_name |      |     | -> proc C
       +-----------+                        | dev_desc | ---->|     | -> proc C
       |           |                        | dev_def  | ---+ |     | -> proc C
       +-----------+                        | drv_def  | -+ | |     | -> proc C
       |           |                        +----------+  | | |     | -> proc C
       +-----------+                        | dev_name +  | | |     | -> proc C
       |           |                        | dev_desc |  | | |     | -> proc C
       +-----------+                        | dev_def  |  | | +-----+
       |           |                        | drv_def  |  | |
       +-----------+                        +----------+  | |
       |           |                        | dev_name |  | | +-----+
       +-----------+                        | dev_desc |  | +>|     |
       |           |                        | dev_def  |  |   |     |
       +-----------+                        | drv_def  |  |   +-----+
       |           |                        +----------+  |
       +-----------+                                      |   +-----+
       |           |                                      +-->|     |
       +-----------+                                          |     |
       |           |                                          +-----+
====== +-----------+ ==========================================================
       |           |                                  Descriptions des
       +-----------+                                  drivers/protocoles/
       |           |                                  services/drivers volumes
       +-----------+
       |           |     Drv_link          Drv_desc
       +-----------+     +----------+      +----------+
       |           | --> | drv_desc | ---> | drv_name |
       +-----------+     | dev_link | -+   | typdrv   |
       |           |     | nb_dev   |  |   | aut      |
       +-----------+     +----------+  |   | trans    | --> Table transitions
       |           |     Une structure |   | lim      |
       +-----------+     Drv_link      |   | etat     |
       |           |                   |   | reqevt   | --> proc C
       +-----------+                   |   | respevt  | --> proc C
       |     0     |                   |   | init     | --> proc C
       +-----------+                   |   | nbvl     |
                                       |   | nbcon    |
                                       |   +----------+
  Tableau d'adresses                   |
  de structures Drv_link               |   Dev_link[]
                                       |   +----------+
                                       +-> | dev_name |      +-----+
                                           | dev_desc | ---->|     | -> proc C
                                           | dev_def  | ---+ |     | -> proc C
                                           | drv_def  | -+ | |     | -> proc C
                                           +----------+  | | |     | -> proc C
                                           | dev_name |  | | +-----+
                                           | dev_desc |  | |
                                           | dev_def  |  | |
                                           | drv_def  |  | | +-----+
                                           +----------+  | +>|     |
                                           | dev_name |  |   |     |
                                           | dev_desc |  |   +-----+
                                           | dev_def  |  |
                                           | drv_def  |  |
                                           +----------+  |
                                                         |   +-----+
                                                         +-->|     |
                                                             |     |
                                                             +-----+

#END --------------------------------------------------------------------------



    3    Fichiers include
    _____________________
*/

#include <stdtyp.l>                    /* Definitions et constantes Cojyp.   */
#include <moteur.h>                    /* S_evt                              */
#include <emu32.h>                     /* E32_ctx                            */
#include <board.h>                     /* Structures drivers VMIO            */
#include <drv.h>                       /* Structures interface haut-niveau   */
#include <vmiodbg.h>                   /* Note_*, ...  debug vmio            */
#include <memoire.h>                   /* Structures Ram                     */
#include <genio.h>                     /* Const_ctx                          */
#include <drv_rcu.h>                   /* RCU_*                              */
#include <drv_i2c.h>                   /* Eep_devdef                         */
#include <drv_int.h>                   /* Driver INT                         */
#include <keycode.h>                   /* Mnemoniques codes de touches.      */
#include <req_lib.h>                   /*                                    */
#include "xdrv_asy.h"                  /* XXX                                */
#include <stm32.h>                     /* Gpio_def                           */
#include <drv_srf.h>                   /* Srf_def, Srf_ctx                   */
#include <drv_led.h>                   /* Srf_def, Srf_ctx                   */
#include <drv_gpio.h>                  /* GPIO                               */
#include <mcu_desc.h>                  /* Defines and registers of STM32L152 */

typedef struct                         /* Contexte d'interface d'un IOD/MODD */
  BEGIN                                /* ---------------------------------- */
    USHORT     node                  ; /* Noeud distant                      */
    USHORT     aut                   ; /* Numero d'automate distant          */
    USHORT     vl                    ; /* Numero de voie logique distante    */
    USHORT     sessid                ; /* Identifiant de session (filtre)    */
    USHORT    *sub_sessid            ; /* Identifiants de session [255]      */
    USHORT    *sub_vl                ; /* Voies logiques          [255]      */
  END          Itf_ctx               ; /* ---------------------------------- */

typedef struct                         /* Contexte d'un Driver/Proto/Service */
  BEGIN                                /* ---------------------------------- */
    Itf_ctx   *itfctx                ; /* Table des contextes Itf_ctx.       */
    USHORT     nbiod                 ; /* Nombre d'IOD                       */
    USHORT     nbmodd                ; /* Nombre de MODD                     */
    UCHAR      sessid_cond_act[21]   ; /* b7:condition, b2-b0: action        */
  END          Drv_itf_ctx           ; /* ---------------------------------- */


/*
    4    Variables externes ( importees )
    _____________________________________

    4.1  Drivers systemes
    _____________________


         La table des drivers VMIO est une liste, un tableau d'adresses de
structures Sdrv_link/Drv_link. Pour les drivers systemes, ce sont des
structures "Sdrv_link" qui sont utilisees. Les structures de type "Sdrv_link"
sont dependantes des boards. On va donc trouver ces structures dans le present
fichier. Chaque structure de type "Sdrv_link" contient un pointeur vers une
structure de type "Sdrv_desc".  Ces dernieres ne sont pas definies dans le
present module, on on trouve ici les references vers ces structures externes "
Sdrv_desc".


         Une structure "Sdrv_desc" est l'interface que tout driver systeme
fournit. Cette structure contient quatre champs. L'adresse d'une chaine, qui
est le nom de ce driver, un entier "typdrv" qui vaut 0 pour tous les drivers
systemes, l'adresse "init" d'une procedure d'initialisation et l'adresse "end"
d'une procedure de terminaison. Chaque source C qui constitue un driver systeme
contient sa propre structure de type "Sdrv_desc" .

*/

extern Sdrv_desc sdrv_sysm3          ; /* Description driver SYSTEM          */
extern Sdrv_desc sdrv_timarmv7m      ; /* Description driver TIME            */
extern Sdrv_desc sdrv_ram            ; /* Description driver RAM             */
extern Sdrv_desc sdrv_int            ; /* Description driver INTERRUPTION    */
extern Sdrv_desc sdrv_vmio           ; /* Description driver VMIO            */
extern Sdrv_desc sdrv_gpio           ; /* Description driver GPIO            */
extern Sdrv_desc sdrv_asy            ; /* Description driver ASY             */
extern Sdrv_desc sdrv_rcu            ; /* Description driver RCU             */
extern Sdrv_desc sdrv_rslv           ; /* Description driver RESOLVE         */

/*

         Les structures de type "Dev_link" sont dependantes des boards, et
sont donc definies dans le present fichier. Le champ "dev_desc" de ces
structures peut parfois pointer sur des donnees definies a l'exterieur du
present fichier, le plus souvent dans le source d'un driver de "bas niveau".
On trouve ici les references externes des donnees qui sont pointees par un
champ "dev_desc" d'une structure de type "Dev_link".

*/

extern IWORD     int_desc_nvic       ; /* Device INT bas-niveau pour L152.   */
extern IWORD     rcu_desc_stm32      ; /* Device RCU bas-niveau pour STM32.  */


/*
    4.2  Drivers caracteres
    _______________________


         La table des drivers VMIO est une liste, un tableau d'adresses de
structures Sdrv_link/Drv_link. Pour les drivers caracteres, ce sont des
structures "Drv_link" qui sont utilisees. Les structures de type "Drv_link"
sont dependantes des boards. On va donc trouver ces structures dans le present
fichier. Chaque structure de type "Drv_link" contient un pointeur vers une
structure de type "Drv_desc". Ces dernieres ne sont pas definies dans le
present module, on on trouve ici les references vers ces structures externes
"Drv_desc".


         Une structure "Drv_desc" est l'interface que tout driver caractere,
tout protocole et tout service fournit. Les deux premiers champs sont les
memes que ceux d'une structure "Sdrv_desc", mais la structure "Drv_desc"
contient beaucoup plus de champs que "Sdrv_desc". Chaque source C qui constitue
un driver caractere contient sa propre structure de type "Sdrv_desc" .

*/

extern Drv_desc  drv_desc_asy        ; /* Description driver asynchrone      */
extern Drv_desc  drv_desc_led        ; /* Description driver LED             */
extern Drv_desc  drv_desc_gpio       ; /* Description driver GPIO            */
extern Drv_desc  drv_desc_kbditf     ; /* Description driver KBDITF          */


/*
         Les structures de type "Dev_link" sont dependantes des boards, et
sont donc definies dans le present fichier. Le champ "dev_desc" de ces
structures peut parfois pointer sur des donnees definies a l'exterieur du
present fichier, le plus souvent dans le source d'un driver de "bas niveau".
On trouve ici les references externes des donnees qui sont pointees par un
champ "dev_desc" d'une structure de type "Dev_link".

*/

extern IWORD     asy_desc_stm32      ; /* Device ASY bas-niveau pour STM32.  */
extern IWORD     gpio_desc_stm32     ; /* Device GPIO bas-niveau.            */


/*
    4.3  Drivers blocs
    __________________

*/

/*
    4.4  Drivers volumes
    ____________________


         La table des drivers du VMIO, qui est une liste d'adresses de
structures de type Sdrv_link/Drv_link, contient un pointeur vers une structure
de type Drv_link pour chaque driver de volume. Dans cette structure, qui
contient trois champs (drv_desc, dev_link et nb_dev) seul le premier est
utilise dans le cas d'une driver de volume. Ce champ "drv_desc" est l'adresse
d'une structure de type "Drv_desc", qui est normalement definie dans le
source C qui constitue le driver de volume. On trouve donc ici les references
externes aux structures "Drv_desc" des drivers de volumes.

*/


/*
    4.5  Protocoles
    _______________


         La table des drivers du VMIO, qui est une liste d'adresses de
structures de type Sdrv_link/Drv_link, contient un pointeur vers une structure
de type Drv_link pour protocole. Comme pour un driver de volume, seul le
premier des trois champs (drv_desc, dev_link et nb_dev) est utilise dans le cas
d'un protocole. Ce champ "drv_desc" est l'adresse d'une structure de type
"Drv_desc", qui est normalement definie dans le source C qui constitue le
driver de volume. On trouve donc ici les references externes aux structures
"Drv_desc" des drivers de volumes.

*/


/*
    4.6  Services
    _____________

*/


/*
    4.7  Procedures des handlers d'interruptions
    ____________________________________________


          On trouvera en 5.4 une table nommee "ithand", qui contient les
adresses de tous les handlers d'interruptions. On trouve ici les references
externes a ces procedures handlers d'interruptions.


         Les valeurs effectives des parametres de chacune des procedures
doivent etre installes par la procedure appelee.

*/

extern IWORD moteur_s                          (IWORD,IWORD                 );
extern IWORD evtto_s                           (IWORD,IWORD                 );
extern IWORD handlerTIM_armv7m                 (IWORD,IWORD                 );
extern IWORD handlerGPIO_stm32                 (IWORD,IWORD                 );

extern IWORD handlerTOASY_stm32                (IWORD,IWORD                 );
extern IWORD handlerASY_stm32                  (IWORD,IWORD                 );
extern IWORD handlerdmaASY_stm32               (IWORD,IWORD                 );

extern IWORD handlerRCU_stm32                  (IWORD,IWORD                 );


/*
    4.8  Procedures vides
    _____________________

         A chaque fois qu'une interface est definie, on peut considerer
qu'un champ vide est materialise par une adresse invalide ou qu'un champ
vide est materialise par une procedure vide. On prend ici le parti de ne
pas tester la validite d'une adresse avant de l'utiliser lorsqu'elle
est lue depuis une interface. Les interfaces definies dans ce fichier
utilisent donc l'adresse d'une procedure vide lorsqu'elles ne proposent
pas une fonctionnalite prevue par une interface. Attention, on peut
aussi definir une procedure vide qui renvoie une erreur.

*/

IWORD PROCEDURE sdrv_empty(drv) IWORD drv; { return(0); }


/*
    4.9 Procedures utilisees par les drivers specifiques
    ____________________________________________________
*/

static IWORD board_init                (void                                 );
static void  infoLED                   (IWORD,const CHAR**,IWORD*,
                                        const CHAR***,
                                        const UCHAR**,const UCHAR**          );
static void  setLED                    (IWORD,IWORD,IWORD                    );



/*
    4.10 Variables globales
    _______________________
*/

const UINT cnt10usec = 39     ; /* See WAIT10US in boot_stm32.c              */

/*
    5    Definitions des drivers systemes
    _____________________________________

    5.1 Driver SYSTEM
    _________________

         Le driver systeme configure les horloges du CPU et du bus.
*/

static const UINT sysdef[] =           /* Parametres du systeme              */
  BEGIN                                /* ---------------------------------- */
#ifdef LEGACY
    16000000                        ,  /* SYSCLK                             */
     8000000                        ,  /* HCLK    (Horloge bus AHB)          */
     2000000                        ,  /* PCLK1   (Horloge bus APB1)         */
     2000000                        ,  /* PCLK2   (Horloge bus APB2)         */
#else
    32000000                        ,  /* SYSCLK                             */
    16000000                        ,  /* HCLK    (Horloge bus AHB)          */
    16000000                        ,  /* PCLK1   (Horloge bus APB1)         */
    16000000                        ,  /* PCLK2   (Horloge bus APB2)         */
#endif
    0x03008007                      ,  /* Config ahben : DMA1&2 (3<<24),     */
                                       /* flash (1<<15), GPIOA,B&C (7<<0)    */
    0x00020000                      ,  /* Config apb1en : USART2 (1<<17)     */
    0x00004000                      ,  /* Config apb2en : USART1 (1<<14)     */
/* @TODO Remplacer ces champs par un appel de la proc sys_ena + supprimer    */
/* l'etape 5 de la proc init_sysm3.                                          */
  END                                ; /* ---------------------------------- */

static const Dev_link dev_sys =        /* Device SYSTEM                      */
  BEGIN                                /* ---------------------------------- */
    "SYS_L152RE"                    ,  /* Nom du device                      */
    (const void *)HNULL             ,  /* Description du device              */
    sysdef                          ,  /* Parametres du systeme              */
    (const void *)HNULL             ,  /* Pas de parametres du driver        */
  END                                ; /* ---------------------------------- */

static const Sdrv_link sdrv_l_sys    = /* Driver SYSTEM                      */
  BEGIN                                /* ---------------------------------- */
    &sdrv_sysm3                     ,  /* Ad. interface  du  driver          */
    &dev_sys                        ,  /* Ad. interfaces des devices         */
    1                               ,  /* 1 device 'SYSTEM'                  */
  END                                ; /* ---------------------------------- */


/*
    5.2 Driver BOARD
    ________________

         On ne fait rien du tout dans le board.

*/

static const Sdrv_desc sdrv_board =    /* Driver systeme de ce board.        */
  BEGIN                                /* ---------------------------------- */
    "BOARD_NUCLEO-64"               ,  /* Nom du driver.                     */
    DRV_SYS                         ,  /* Type du driver.                    */
    board_init                      ,  /* Procedure d'initialisation         */
    sdrv_empty                      ,  /* Procedure de terminaison           */
  END                                ; /* ---------------------------------- */

static const Sdrv_link sdrv_l_board =  /* Driver BOARD Rev0                  */
  BEGIN                                /* ---------------------------------- */
    &sdrv_board                     ,  /* Ad. interface  du  driver          */
    HNULL                           ,  /* Ad. interfaces des devices         */
    1                               ,  /* 1 device 'BOARD'                   */
  END                                ; /* ---------------------------------- */


/*
    5.3  Driver MEMORY
    __________________

*/


/*
    5.3.2  Les allocations du VMOS
    ______________________________

         La zone memoire du VMOS est celle qu'il utilise pour toutes les
allocations dynamiques du VMOS. 

*/

static const Ram_def_vmos ram_def_vmos  ; /* Parametrage memoire VMOS        */


/*
    5.3.3  Les services
    ___________________

         Il n'y a pas de services pour l'instant.

*/

static const Ram_def_srv ram_def_srv   ;  /* Parametrage memoires services   */



/*
    5.3.4 Les buffers telecom
    _________________________

*/

/*****************************************************************************/
/* RAM pour buffer telecoms : 24 Ko                                          */
/* La valeur de cette adresse est renseignee par le linker!                  */
/*****************************************************************************/

extern UINT      ad_buf_part0           ; /* Importe depuis le linker        */
extern UINT      lg_buf_part0           ; /* Importe depuis le linker        */
extern UINT      end_buf_part0          ; /* Importe depuis le linker        */


const Ram_def_part ram_def_part[] =       /* Parametrage memoire Telecom     */
  BEGIN                                   /* ------------------------------- */

/*****************************************************************************
 * 1ERE et DERNIERE PARTITION                                                *
 *****************************************************************************/
    BEGIN                                 /* Definition de partition ------- */

    1                                  ,  /* Un seul segment memoire         */
    /* PJB : Pourquoi cette valeur? .........................................*/
    4576                               ,  /* Taille d'un bigbuff             */

    (UCHAR*)&  ad_buf_part0             , /* Ad. de debut du segment         */
    (UCHAR*)& end_buf_part0 - 1         , /* Ad. de fin   du segment         */
    (UINT  )&  lg_buf_part0             , /* Taille en octet du segment.     */
    (USHORT)1                             /* Segment en cache.               */

    END                                , /* -------------------------------- */

  END                                   ; /* ------------------------------- */


/*
        5.3.5 Table de definition
        _________________________
*/

const Ram_def_dev ram_def_dev        ; /* Importe memoires des devices       */

const Ram_def ram_def =                /* Parametrage de la RAM              */
  BEGIN                                /* ---------------------------------- */
    HNULL                           ,  /* Memoire pages teletexte            */
    &ram_def_vmos                   ,  /* Memoire allocations VMOS           */
    &ram_def_srv                    ,  /* Memoire services                   */
    &ram_def_dev                    ,  /* Memoire devices                    */
    ram_def_part                    ,  /* Memoire partitions buffers         */
    1                               ,  /* Deux partitions de buffers         */
  END                                ; /* ---------------------------------- */

static const Dev_link dev_ram =        /* Configurations memoire RAM         */
  BEGIN                                /* ---------------------------------- */
    "RAM"                           ,  /* Nom du device                      */
    HNULL                           ,  /* Sur systeme embarque               */
    (void *)&ram_def                ,  /* Parametrage de la RAM              */
    HNULL                           ,  /* Une seule configuration de RAM     */
  END                                ; /* ---------------------------------- */

static const Sdrv_link sdrv_l_mem =    /* Driver RAM                         */
  BEGIN                                /* ---------------------------------- */
    &sdrv_ram                       ,  /* Ad. interface  du  driver          */
    &dev_ram                        ,  /* Ad. interface  des devices         */
    1                               ,  /* 1 device 'RAM'                     */
  END                                ; /* ---------------------------------- */


/*
    5.4 Driver INTERRUPT
    ____________________
*/

/* Numerotation des handlers ............................................... */

#define H_MOTEURS       0
#define H_TIME          1
#define H_EVTTOS        2
#define H_ASYT0         3
#define H_ASYT1         4
#define H_ASYT2         5
#define H_GPIO00        6
#define H_GPIO01        7
#define H_GPIO02        8
#define H_GPIO03        9
#define H_GPIO04       10
#define H_GPIO05       11
#define H_GPIO06       12
#define H_GPIO07       13
#define H_GPIO08       14
#define H_GPIO09       15
#define H_GPIO10       16
#define H_GPIO11       17
#define H_GPIO12       18
#define H_GPIO13       19
#define H_GPIO14       20
#define H_GPIO15       21
#define H_ASY0         22
#define H_ASY1         23
#define H_ASY2         24
#define H_ASY0_DMATX   25
#define H_ASY0_DMARX   26
#define H_RCU          27

/* Chainage des handlers ................................................... */

static const Ithand_t ithand[] =                        /* Chainage handlers */
  BEGIN                                                 /* ----------------- */
    0xFFFFFFFF             , moteur_s          ,  0, 0, /*  0 : H_MOTEURS    */
                                                        /*                   */
    (H_EVTTOS<<24)|0xFFFFFF, handlerTIM_armv7m ,  0, 0, /*  1 : H_TIME       */
    (H_ASYT0 <<24)|0xFFFFFF, evtto_s           ,  0, 0, /*  2 : H_EVTTOS     */
    (H_ASYT1 <<24)|0xFFFFFF, handlerTOASY_stm32,  0, 0, /*  3 : H_ASYT0      */
    (H_ASYT2 <<24)|0xFFFFFF, handlerTOASY_stm32,  1, 0, /*  4 : H_ASYT1      */
    0xFFFFFFFF             , handlerTOASY_stm32,  2, 0, /*  5 : H_ASYT2      */
                                                        /*                   */
    0xFFFFFFFF             , handlerGPIO_stm32 ,  0, 0, /*  6 : H_GPIO00     */
    0xFFFFFFFF             , handlerGPIO_stm32 ,  1, 0, /*  7 : H_GPIO01     */
    0xFFFFFFFF             , handlerGPIO_stm32 ,  2, 0, /*  8 : H_GPIO02     */
    0xFFFFFFFF             , handlerGPIO_stm32 ,  3, 0, /*  9 : H_GPIO03     */
    0xFFFFFFFF             , handlerGPIO_stm32 ,  4, 0, /* 10 : H_GPIO04     */
                                                        /*                   */
    (H_GPIO06<<24)|0xFFFFFF, handlerGPIO_stm32 ,  5, 0, /* 11 : H_GPIO05     */
    (H_GPIO07<<24)|0xFFFFFF, handlerGPIO_stm32 ,  6, 0, /* 12 : H_GPIO06     */
    (H_GPIO08<<24)|0xFFFFFF, handlerGPIO_stm32 ,  7, 0, /* 13 : H_GPIO07     */
    (H_GPIO09<<24)|0xFFFFFF, handlerGPIO_stm32 ,  8, 0, /* 14 : H_GPIO08     */
    0xFFFFFFFF             , handlerGPIO_stm32 ,  9, 0, /* 15 : H_GPIO09     */
                                                        /*                   */
    (H_GPIO11<<24)|0xFFFFFF, handlerGPIO_stm32 , 10, 0, /* 16 : H_GPIO10     */
    (H_GPIO12<<24)|0xFFFFFF, handlerGPIO_stm32 , 11, 0, /* 17 : H_GPIO11     */
    (H_GPIO13<<24)|0xFFFFFF, handlerGPIO_stm32 , 12, 0, /* 18 : H_GPIO12     */
    (H_GPIO14<<24)|0xFFFFFF, handlerGPIO_stm32 , 13, 0, /* 19 : H_GPIO13     */
    (H_GPIO15<<24)|0xFFFFFF, handlerGPIO_stm32 , 14, 0, /* 20 : H_GPIO14     */
    0xFFFFFFFF             , handlerGPIO_stm32 , 15, 0, /* 21 : H_GPIO15     */
                                                        /*                   */
    0xFFFFFFFF             , handlerASY_stm32  ,  0, 0, /* 22 : H_ASY0       */
    0xFFFFFFFF             , handlerASY_stm32  ,  1, 0, /* 23 : H_ASY1       */
    0xFFFFFFFF             , handlerASY_stm32  ,  2, 0, /* 24 : H_ASY2       */
    0xFFFFFFFF             , handlerdmaASY_stm32, 0, 0, /* 25 : H_ASY0_DMATX */
    0xFFFFFFFF             , handlerdmaASY_stm32, 0, 1, /* 26 : H_ASY0_DMARX */
                                                        /*                   */
    0xFFFFFFFF             , handlerRCU_stm32  ,  1, 0, /* 27 : H_RCU        */
                                                        /*                   */
    0                                                   /* MARQUEUR DE FIN   */
  END                                                 ; /* ----------------- */

/* Raccorde les tetes de chainage des handlers ............................ */

static const Ithead_t ithead[] =       /* Association code d'IT <-> handler  */
  BEGIN                                /* ---------------------------------- */

/* Priorite 0, on y raccorde les fils d'IT device inutilisees .............. */
#define L_ITPRI0  0

/* Priorite 1, ce fil est commande par logiciel ............................ */
#define L_ITPRI1  L_ITPRI0 + 1

    0      , 1 , H_MOTEURS          ,  /* IRQ 0 , interruption logicielle    */

/* Priorite 2 .............................................................. */
#define L_ITPRI2  L_ITPRI1 + 5

    37     , 1 , H_ASY0             ,  /* IRQ37 , USART1                     */
    38     , 1 , H_ASY1             ,  /* IRQ38 , USART2                     */
    39     , 1 , H_ASY2             ,  /* IRQ39 , USART3                     */
    14     , 1 , H_ASY0_DMATX       ,  /* IRQ14 , DMA1 canal 4 : USART1 TX   */
    15     , 1 , H_ASY0_DMARX       ,  /* IRQ15 , DMA1 canal 5 : USART1 RX   */

/* Priorite 3 .............................................................. */
#define L_ITPRI3  L_ITPRI2 + 7

     6     , 1 , H_GPIO00            , /* IRQ 6 , GPIO 0                     */
     7     , 1 , H_GPIO01            , /* IRQ 7 , GPIO 1                     */
     8     , 1 , H_GPIO02            , /* IRQ 8 , GPIO 2                     */
     9     , 1 , H_GPIO03            , /* IRQ 9 , GPIO 3                     */
    10     , 1 , H_GPIO04            , /* IRQ10 , GPIO 4                     */
    23     , 1 , H_GPIO05            , /* IRQ23 , GPIO 5 a GPIO 9            */
    40     , 1 , H_GPIO10            , /* IRQ40 , GPIO 10 a GPIO 15          */

/* Priorite 4 .............................................................. */
#define L_ITPRI4  L_ITPRI3 + 1

    46     , 1 , H_RCU              ,  /* IRQ46 , TIMER 5 : RCU              */

/* Priorite 5 .............................................................. */
#define L_ITPRI5  L_ITPRI4 + 0

/* Priorite 6 .............................................................. */
#define L_ITPRI6  L_ITPRI5 + 1

    -1     , 1 , H_TIME             ,  /* SysTick, decompteur systeme.       */

/* Priorite 7 .............................................................. */
#define L_ITPRI7  L_ITPRI6 + 0

  END                                ; /* ---------------------------------- */


static const IWORD itidx[] =           /* Priorites des interruptions        */
  BEGIN                                /* ---------------------------------- */
    0        , L_ITPRI0 , L_ITPRI1 ,   /* Delimitation des priorites dans    */
    L_ITPRI2 , L_ITPRI3 , L_ITPRI4 ,   /* la table "it_head"                 */
    L_ITPRI5 , L_ITPRI6 , L_ITPRI7 ,   /*                                    */
  END                                ; /* ---------------------------------- */


/* Definition du driver Interruption ....................................... */

static const Itdev_t itdev =           /* Definition locale pour INT.        */
  BEGIN                                /* ---------------------------------- */
    itidx                           ,  /* Priorites des interruptions        */
    ithead                          ,  /* Association code d'IT <-> handler  */
    ithand                          ,  /* Handlers d'interruption            */
  END                                ; /* ---------------------------------- */

static const Dev_link dev_int =        /* Device INTERRUPT                   */
  BEGIN                                /* ---------------------------------- */
    "INT_L152"                      ,  /* Nom du device                      */
    (const void *)&int_desc_nvic    ,  /* Controleur d'interruption NVIC     */
    (const void *)&itdev            ,  /* Ad. structure parametres  device   */
    (const void *)HNULL             ,  /* Pas de parametres driver/dev       */
  END                                ; /* ---------------------------------- */

static const Sdrv_link sdrv_l_int =    /* Driver INTERRUPT                   */
  BEGIN                                /* ---------------------------------- */
    &sdrv_int                       ,  /* Ad. interface  du  driver          */
    &dev_int                        ,  /* Ad. interfaces des devices         */
    1                               ,  /* 1 device 'INT'                     */
  END                                ; /* ---------------------------------- */


/*
    5.5 Driver TIME
    _______________
*/

static const Dev_link dev_tim =        /* Device TIME                        */
  BEGIN                                /* ---------------------------------- */
    "TIM_L152"                       , /* Nom du device                      */
    (const void *)HNULL              , /* Pas de description device          */
    (const void *)HNULL              , /* Pas de parametres du peripherique  */
    (const void *)HNULL              , /* Pas de parametres du systeme       */
  END                                ; /* ---------------------------------- */

static const Sdrv_link sdrv_l_tim =    /* Driver TIME                        */
  BEGIN                                /* ---------------------------------- */
    &sdrv_timarmv7m                  , /* Ad. interface  du driver           */
    &dev_tim                         , /* Ad. interfaces des devices         */
    1                                , /* 1 device 'TIME'                    */
  END                                ; /* ---------------------------------- */



/*
    5.6 Driver VMIO
    _______________
*/

static const Sdrv_link sdrv_l_vmio =   /* Driver systeme VMIO                */
  BEGIN                                /* ---------------------------------- */
    &sdrv_vmio                      ,  /* Driver systeme generique           */
    HNULL                           ,  /* Pas de configuration               */
    0                               ,  /* Aucun device                       */
 END                                 ; /* ---------------------------------- */



/*
    5.7  Driver PIO
    _______________

         On fait le choix de considerer qu'il n'y a qu'un device GPIO. On
pourrait aussi bien dire qu'il y en a 8 (1 par banque), mais ca n'apporte
aucun benefice.

*/

/* Contextes GPIO ...........................................................*/

#define GPIO_NBCTL          1          /* Only one device.                   */
#define GPIO_NBPIN         48          /* Number of pins.                    */
#define GPIO_NBVL     (1+1+48)         /* Total number of logical channels.  */

Gpioctl    gpioctl[GPIO_NBCTL]       ; /* Controller's context table.        */
Gpiopin    gpiopin[GPIO_NBPIN]       ; /* Pins' contexts table.              */

USHORT       gpio_states[GPIO_NBVL]  ; /* Logical channels states.           */
const USHORT gpio_nbvl = GPIO_NBVL   ; /* Number of logical channels.        */

Itf_ctx gpio_itfctx[1+GPIO_NBCTL] = {0} ;

/* Hardware definitions .....................................................*
 * The pins PA2 and PA3 are connected to ST-LINK MCU (see "UM1724 User       *
 * manual" (file "STM32NUCLEO_User_manual.pdf"), revision 12 (december       *
 * 2017), chapter "6.8 USART communication").                                */

Gpio_pindef pindefs[] =                /* Specific init.                     */
  BEGIN                                /* ---------------------------------- */
    "PA0" , 1, "TIM345" , 0, GPIO_PU , /* PA0  TMR5 RCU                      */
                                       /* PA1  User button 2                 */
    "PA2" , 1, "UART123", 0, GPIO_PU , /* PA2  UART2-TX (ST LINK)            */
    "PA3" , 1, "UART123", 0, GPIO_PU , /* PA3  UART2-RX (ST LINK)            */
    "PA7" , 1, "GPIO"   , 0, 0       , /* PA7  LED01 RED                     */
    "PA8" , 1, "GPIO"   , 0, 0       , /* PA8  LED03 YELLOW                  */
    "PA9" , 1, "UART123", 0, GPIO_PU , /* PA9  UART1-TX                      */
    "PA10", 1, "UART123", 0, GPIO_PU , /* PA10 UART1-RX                      */
    "PA11", 1, "GPIO"   , 0, GPIO_PD , /* PA11 UART1-CTS: down: forcer emissi*/
    "PA12", 1, "GPIO"   , 0, 0       , /* PA12 UART1-RTS                     */
    "PA13", 1, "SYS"    , 0, GPIO_PU , /* PA13 JTMS-SWDIO  JTAG              */
    "PA14", 1, "SYS"    , 0, GPIO_PD , /* PA14 JTCK-SWCLK  JTAG              */
    "PA15", 1, "SYS"    , 0, GPIO_PU , /* PA15 JTDI        JTAG              */
                                       /* PB0  User button 3                 */
                                       /* PB5  User button 1                 */
    "PC7" , 1, "GPIO"   , 0, 0       , /* PC7  LED02 GREEN                   */
    "PC10", 1, "UART123", 0, GPIO_PU , /* PC10 UART3-TX                      */
    "PC11", 1, "UART123", 0, GPIO_PU , /* PC11 UART3-RX                      */
                                       /* PC13 B1 USER (blue button)         */
    HNULL                            , /* END OF TABLE                       */
  END                                ; /* ---------------------------------- */

Gpio_def gpio_def =                    /* High-level driver definition.      */
  BEGIN                                /* ---------------------------------- */
    pindefs                          , /* Table of specific init.            */
  END                                ; /* ---------------------------------- */


static const Dev_link dev_gpio =       /* Device GPIO                        */
  BEGIN                                /* ---------------------------------- */
    "DEV0"                          ,  /* Nom du device                      */
    (void*)&gpio_desc_stm32         ,  /* Interface bas-niveau.              */
    (void*)&gpio_def                ,  /* Description board                  */
    HNULL                           ,  /* Pas de description driver          */
  END                                ; /* ---------------------------------- */

static const Sdrv_link sdrv_l_gpio =   /* Driver GPIO                        */
  BEGIN                                /* ---------------------------------- */
    &sdrv_gpio                      ,  /* Driver systeme                     */
    &dev_gpio                       ,  /* Configuration des devices GPIO     */
    GPIO_NBCTL                      ,  /* 1 device GPIO                      */
  END                                ; /* ---------------------------------- */


/*
    5.9 Driver ASY
    ______________

*/

/* Contextes ASY ............................................................*/

#define NUM_ASY              3         /* Number of used UARTs               */
#define BUF_SIZE            64         /* Circular buffer size               */
#define NUM_OF_SEG           2         /* Number of segments of circular buff*/

Asy_ctx       asy_ctx   [NUM_ASY]    ; /* Contextes.                         */

static const
    Asy_stm32_def asy_def[NUM_ASY] =   /* Low level device definition        */
  BEGIN                                /* ---------------------------------- */
                                       /*                                    */
    0                                , /*** USART1 ***************************/
    0                                , /* DMA number                         */
    3                                , /* USART1 TX channel                  */
    4                                , /* USART1 RX channel                  */
                                       /* Description des GPIO des signaux   */
                                       /* port, pin, logique; inutilise = -1 */
    0x00, 11, 1                      , /* CTS     USART1 CTS (PA11)          */
    0xFF,  0, 0                      , /* DSR                                */
    0xFF,  0, 0                      , /* RI                                 */
    0xFF,  0, 0                      , /* DCD                                */
    0xFF,  0, 0                      , /* DTR                                */
    0x00, 12, 1                      , /* RTS     USART1 RTS (PA12)          */
    0x00, 10, 0                      , /* RXB     USART1 RX  (PA10)          */
    0x00,  9, 0                      , /* TXB     USART1 TX  (PA09)          */
                                       /*                                    */
    1                                , /*** USART2 ***************************/
    0xFF                             , /* DMA number (not used)              */
    0xFF                             , /* USART2 TX channel (not used)       */
    0xFF                             , /* USART2 RX channel (not used)       */
                                       /* Description des GPIO des signaux   */
                                       /* port, pin, logique; inutilise = -1 */
    0xFF,  0, 0                      , /* CTS                                */
    0xFF,  0, 0                      , /* DSR                                */
    0xFF,  0, 0                      , /* RI                                 */
    0xFF,  0, 0                      , /* DCD                                */
    0xFF,  0, 0                      , /* DTR                                */
    0xFF,  0, 0                      , /* RTS                                */
    0x00,  3, 0                      , /* RXB     USART2 RX (PA03) ST LINK   */
    0x00,  2, 0                      , /* TXB     USART2 TX (PA02) ST LINK   */
                                       /*                                    */
    2                                , /*** USART3 ***************************/
    0xFF                             , /* DMA number (not used)              */
    0xFF                             , /* USART3 TX channel (not used)       */
    0xFF                             , /* USART3 RX channel (not used)       */
                                       /* Description des GPIO des signaux   */
                                       /* port, pin, logique; inutilise = -1 */
    0xFF,  0, 0                      , /* CTS                                */
    0xFF,  0, 0                      , /* DSR                                */
    0xFF,  0, 0                      , /* RI                                 */
    0xFF,  0, 0                      , /* DCD                                */
    0xFF,  0, 0                      , /* DTR                                */
    0xFF,  0, 0                      , /* RTS                                */
    0x02, 11, 0                      , /* RXB     USART3 RX (PC11)           */
    0x02, 10, 0                      , /* TXB     USART3 TX (PC10)           */
                                       /*                                    */
  END                                ; /* ---------------------------------- */

static const
    Asy_drvdef asy_drvdef[NUM_ASY] =   /* Device definition                  */
  BEGIN                                /* ---------------------------------- */
                                       /*                                    */
    1                                , /* Index du device pour le bas-niveau */
    BR_ALL & ~((1 << BR_50) |          /* Vitesses supportees : vitesses >   */
      (1 << BR_75)  | (1 << BR_110) |  /* 4000000 / 16 / 2^12 / 0,2 (+/-10%  */
      (1 << BR_134) | (1 << BR_150) |  /* d'erreur max) # 305 bauds          */
      (1 << BR_200))                 , /*                                    */
    ST_2 | ST_1                      , /* Only 1 or 2 stop bits              */
    0                                , /* Mode 5 bits non supporte           */
    0                                , /* Mode 6 bits non supporte           */
    PAR_ODD | PAR_EVEN               , /* Mode 7 bits avec parite uniquement */
    PAR_ALL                          , /* Mode 8 bits complet                */
    PAR_NONE                         , /* Mode 9 bits sans parite uniquement */
    0x00                             , /* Rien n'est supporte                */
                                       /*                                    */
    0                                , /* Index du device pour le bas-niveau */
    BR_ALL & ~((1 << BR_50) |          /* Vitesses supportees : vitesses >   */
      (1 << BR_75)  | (1 << BR_110) |  /* 4000000 / 16 / 2^12 / 0,2 (+/-10%  */
      (1 << BR_134) | (1 << BR_150) |  /* d'erreur max) # 305 bauds          */
      (1 << BR_200))                 , /*                                    */
    ST_2 | ST_1                      , /* Only 1 or 2 stop bits              */
    0                                , /* Mode 5 bits non supporte           */
    0                                , /* Mode 6 bits non supporte           */
    PAR_ODD | PAR_EVEN               , /* Mode 7 bits avec parite uniquement */
    PAR_ALL                          , /* Mode 8 bits complet                */
    PAR_NONE                         , /* Mode 9 bits sans parite uniquement */
    OTH_DMA |                          /* DMA supporte en hardware           */
    OTH_CTS                          , /* CTS supporte en hardware           */
                                       /*                                    */
    2                                , /* Index du device pour le bas-niveau */
    BR_ALL & ~((1 << BR_50) |          /* Vitesses supportees : vitesses >   */
      (1 << BR_75)  | (1 << BR_110) |  /* 4000000 / 16 / 2^12 / 0,2 (+/-10%  */
      (1 << BR_134) | (1 << BR_150) |  /* d'erreur max) # 305 bauds          */
      (1 << BR_200))                 , /*                                    */
    ST_2 | ST_1                      , /* Only 1 or 2 stop bits              */
    0                                , /* Mode 5 bits non supporte           */
    0                                , /* Mode 6 bits non supporte           */
    PAR_ODD | PAR_EVEN               , /* Mode 7 bits avec parite uniquement */
    PAR_ALL                          , /* Mode 8 bits complet                */
    PAR_NONE                         , /* Mode 9 bits sans parite uniquement */
    0x00                             , /* Rien n'est supporte                */
                                       /*                                    */
  END                                ; /* ---------------------------------- */

static UCHAR circ_buffer[BUF_SIZE]   ; /* Circular buffer                    */

const Circ_buf circ_buf[NUM_ASY] =     /* Circular buffer definition         */
  BEGIN                                /* ---------------------------------- */
                                       /*                                    */
    &circ_buffer[0]                  , /* Circular buffer base address       */
    BUF_SIZE / NUM_OF_SEG            , /* Segment size in bytes              */
    NUM_OF_SEG                       , /* Total number of segments           */
    1                                , /* Number of next segment to write    */
    0                                , /* Number of next segment to be read  */
    NUM_OF_SEG                       , /* Number of used segments            */
                                       /*                                    */
    HNULL                            , /* Circular buffer base address       */
    0                                , /* Segment size in bytes              */
    0                                , /* Total number of segments           */
    0                                , /* Number of next segment to write    */
    0                                , /* Number of next segment to be read  */
    0                                , /* Number of used segments            */
                                       /*                                    */
  END                                ; /* ---------------------------------- */

static const Dev_link  sdev_asy[] =    /* Device ASY                         */
  BEGIN                                /* ---------------------------------- */
    "ASY L152 0"                     , /* Nom du device                      */
    &asy_desc_stm32                  , /* Description du device UART         */
    (void*)&asy_def[0]               , /* Description du device UART         */
    (void*)&asy_drvdef[0]            , /* Description du device UART         */
                                       /*                                    */
    "ASY L152 1"                     , /* Nom du device                      */
    &asy_desc_stm32                  , /* Description du device UART         */
    (void*)&asy_def[1]               , /* Description du device UART         */
    (void*)&asy_drvdef[1]            , /* Description du device UART         */
                                       /*                                    */
    "ASY L152 2"                     , /* Nom du device                      */
    &asy_desc_stm32                  , /* Description du device UART         */
    (void*)&asy_def[2]               , /* Description du device UART         */
    (void*)&asy_drvdef[2]            , /* Description du device UART         */
  END                                ; /* ---------------------------------- */
 
static const Sdrv_link sdrv_l_asy   =  /* Driver ASY                         */
  BEGIN                                /* ---------------------------------- */
    &sdrv_asy                        , /* Ad. interface  du  driver          */
    sdev_asy                         , /* Ad. interfaces des devices         */
    NUM_ASY                          , /* 5 devices 'ASY'                    */
  END                                ; /* ---------------------------------- */


/*
    5.10 Driver RCU
    _______________

*/

static const UCHAR rcudef[] =          /* Definition device telecommande     */
  BEGIN                                /* ---------------------------------- */
    28                               , /* Numero telecommande par defaut     */
                                       /* 28 = ECEL JU090601                 */
    0x00 | RCU_IR                    , /* RC sur port IR, signal normal      */
    0                                , /* Numero symbole de debut            */
  END                                ; /* ---------------------------------- */

/* @TODO MOVE rcudef from "dev_def" (hardware dependent) to "drv_def".       */
static const Dev_link dev_rcu =        /* Description specifique             */
  BEGIN                                /* ---------------------------------- */
    "RCU_STM32"                      , /* Nom du device                      */
    (void *)&rcu_desc_stm32          , /* Interface globale RCU STM32        */
    (void *)&rcudef                  , /* Definition locale RCU              */
    (void *)HNULL                    , /* Pas de description driver          */
  END                                ; /* ---------------------------------- */

static const Sdrv_link sdrv_l_rcu =    /* Driver RCU                         */
  BEGIN                                /* ---------------------------------- */
    &sdrv_rcu                        , /* Driver systeme generique           */
    &dev_rcu                         , /* Configuration                      */
    1                                , /* 1 device : RCU                     */
  END                                ; /* ---------------------------------- */

 

/*
    5.11 Driver RESOLVE
    ___________________

    Ce driver systeme ne pilote pas de hardware, mais sert a la resolution de
nom d'automate pour l'initialisation des drivers caracteres.
*/

static const Sdrv_link sdrv_l_rslv =   /* Driver RESOLVE                     */
  BEGIN                                /* ---------------------------------- */
    &sdrv_rslv                      ,  /* Ad. interface du driver            */
    HNULL                           ,  /* Ad. interfaces des devices         */
    0                               ,  /* Pas de device pour RESOLVE         */
  END                                ; /* ---------------------------------- */

/*

    6    Definitions des drivers caracteres
    _______________________________________

    6.1  Driver UART
    ________________
*/

/* Contextes UART ...........................................................*/

const USHORT  asy_nbiod= NUM_ASY     ; /* Nombre de device pour le VMK.      */
const USHORT  asy_nbvl = NUM_ASY+1   ; /* Nombre de voies logiques           */
USHORT        asy_etat  [NUM_ASY+1]  ; /* Etat de voie logique               */

Itf_ctx asy_itfctx[1+NUM_ASY] = { 0 } ;

static const Dev_link  cdev_asy[] =    /* Device ASY                         */
  BEGIN                                /* ---------------------------------- */
    "DEV0"                           , /* Nom du device                      */
    &asy_desc_stm32                  , /* Description du device UART         */
    (void*)&asy_def[0]               , /* Description du device UART         */
    (void*)&asy_drvdef[0]            , /* Description du device UART         */
                                       /*                                    */
    "DEV1"                           , /* Nom du device                      */
    &asy_desc_stm32                  , /* Description du device UART         */
    (void*)&asy_def[1]               , /* Description du device UART         */
    (void*)&asy_drvdef[1]            , /* Description du device UART         */
                                       /*                                    */
    "DEV2"                           , /* Nom du device                      */
    &asy_desc_stm32                  , /* Description du device UART         */
    (void*)&asy_def[2]               , /* Description du device UART         */
    (void*)&asy_drvdef[2]            , /* Description du device UART         */
  END                                ; /* ---------------------------------- */

static const Drv_link  cdrv_l_asy =    /* Driver ASY                         */
  BEGIN                                /* ---------------------------------- */
    &drv_desc_asy                    , /* Description du driver              */
    cdev_asy                         , /* Description des devices            */
    NUM_ASY                          , /* Nombre de devices                  */
    HNULL                              /* Interface pipe                     */
  END                                ; /* ---------------------------------- */

/*
    6.2  Driver LED
    _______________
*/

/* Contextes LED ........................................................... */

#define LEDCTRL_NB   1                 /* Nombre de controleurs de LED = 1.  */
#define LED_NB       3                 /* Nombre de LEDs de ce controleur.   */
#define LED_NBVL     1+3+1             /* Nombre total de voies logiques.    */

const USHORT led_nbiod = LEDCTRL_NB  ; /* Nombre de VL controleurs.          */
const USHORT led_nbvl  = LED_NBVL    ; /* Nombre total de VL.                */

Ledctrl_ctx ledctrl_ctx [LEDCTRL_NB] ; /* Contextes des controleurs.         */
Led_ctx     led_ctx     [LED_NB    ] ; /* Contextes des LEDs.                */
USHORT      led_etat    [LED_NBVL  ] ; /* Contextes des LEDs.                */

Itf_ctx led_itfctx[1+LEDCTRL_NB] = { 0 } ;

static const Led_desc led_desc =       /* Description du bas-niveau          */
  BEGIN                                /* ---------------------------------- */
    infoLED                          , /*                                    */
    setLED                           , /*                                    */
  END                                ; /* ---------------------------------- */

static const Dev_link cdev_led =       /* Device LED STM32                   */
  BEGIN                                /* ---------------------------------- */
    "DEV0"                           , /* Nom du device                      */
    &led_desc                        , /* Description device LED 0           */
    HNULL                            , /* Pas de parametres                  */
    HNULL                            , /* Pas de description driver          */
  END                                ; /* ---------------------------------- */

static const Drv_link cdrv_l_led =     /* Driver LED STM32                   */
  BEGIN                                /* ---------------------------------- */
    &drv_desc_led                    , /* Description du driver              */
    &cdev_led                        , /* Description des devices            */
    1                                , /* Nombre de devices                  */
    HNULL                              /* Interface pipe                     */
  END                                ; /* ---------------------------------- */

/* Description des LEDs .................................................... */

static const CHAR *lednames[LED_NB] =  /* Noms des LEDs                      */
  BEGIN                                /* ---------------------------------- */
    "RED0", "GREEN0", "YELLOW0"      , /* Noms des LEDs par couleurs         */
  END                                ; /* ---------------------------------- */

static const UCHAR ledcol[LED_NB] =    /* Couleurs des LEDs                  */
  BEGIN                                /* ---------------------------------- */
    C_RED, C_GREEN , C_YELLOW        , /* LEDs tri-colores.                  */
  END                                ; /* ---------------------------------- */

static const UCHAR ledsta[LED_NB] =    /* Etats des LEDs                     */
  BEGIN                                /* ---------------------------------- */
    ST_OFF, ST_OFF, ST_OFF           , /* Toutes les LEDs sont eteintes.     */
  END                                ; /* ---------------------------------- */

static const UCHAR ledpio[LED_NB][2] = /* Table des GPIOs de controle LEDs   */
  BEGIN                                /* ---------------------------------- */
    0, 7                             , /* LED0 sur PA7                       */
    2, 7                             , /* LED1 sur PC7                       */
    0, 8                             , /* LED2 sur PA8                       */
  END                                ; /* ---------------------------------- */


/*
    6.3  Driver GPIO
    ________________
*/

static const Drv_link cdrv_l_gpio =    /* Driver GPIO                        */
  BEGIN                                /* ---------------------------------- */
    &drv_desc_gpio                  ,  /* Driver systeme                     */
    &dev_gpio                       ,  /* Configuration des devices GPIO     */
    1                               ,  /* 1 device GPIO                      */
  END                                ; /* ---------------------------------- */


/*
    6.4  Driver KBDITF
    __________________
*/

Itf_ctx kbditf_itfctx[1+1] = { 0 } ;

static const Dev_link dev_kbditf[1] =  /* Device KBDITF                      */
  BEGIN                                /* ---------------------------------- */
    "DEV0"                           , /* Nom du device                      */
    HNULL                            , /* Description device                 */
    HNULL                            , /* Pas de parametres                  */
    HNULL                            , /* Pas de description driver          */
  END                                ; /* ---------------------------------- */

static const Drv_link cdrv_l_kbditf  = /* Driver KBDITF                      */
  BEGIN                                /* ---------------------------------- */
    &drv_desc_kbditf                 , /* Driver KBDITF                      */
    dev_kbditf                       , /* Configuration                      */
    1                                , /* 1 device                           */
    HNULL                              /* Interface pipe                     */
  END                                ; /* ---------------------------------- */

/*
    7    Definitions des drivers blocs
    __________________________________


    8    Liste des procedures du module
    ___________________________________


    9    Definitions des drivers protocoles
    _______________________________________


    10   Definitions des drivers services
    _____________________________________


    11   Variables de l'interface (exportes)
    ________________________________________

    11.1 Board
    __________
*/

static const Drv_link * const drv_l152[] =       /* Liste des drivers VMIO   */
  BEGIN                                          /* ------------------------ */

/* Liste des drivers systeme ............................................... */

    (const Drv_link *)&sdrv_l_sys             ,  /*  0 SYSTEM                */
    (const Drv_link *)&sdrv_l_board           ,  /*  1 BOARD                 */
    (const Drv_link *)&sdrv_l_mem             ,  /*  2 MEMORY                */
    (const Drv_link *)&sdrv_l_int             ,  /*  3 INTERRUPT             */
    (const Drv_link *)&sdrv_l_tim             ,  /*  4 TIME                  */
    (const Drv_link *)&sdrv_l_vmio            ,  /*  5 VMIO                  */
    (const Drv_link *)&sdrv_l_gpio            ,  /*  6 GPIO                  */
    (const Drv_link *)&sdrv_l_asy             ,  /*  7 ASY                   */
    (const Drv_link *)&sdrv_l_rcu             ,  /*  8 RCU                   */
    (const Drv_link *)&sdrv_l_rslv            ,  /*  9 RESOLVE (avant carac) */

/* Liste des drivers caracteres ............................................ */

    (const Drv_link *)&cdrv_l_asy             ,  /* 10 ASY                   */
    (const Drv_link *)&cdrv_l_gpio            ,  /* 11 GPIO                  */
    (const Drv_link *)&cdrv_l_led             ,  /* 12 LED                   */
    (const Drv_link *)&cdrv_l_kbditf          ,  /* 13 KBDITF                */

/* Liste des drivers bloc .................................................. */
/* Liste des drivers de volume ............................................. */
/* Liste des protocoles .................................................... */
/* Liste des services ...................................................... */
/* Marque de fin de liste .................................................. */

    HNULL                                     ,  /* FIN DE LISTE             */
  END                                          ; /* ------------------------ */

Board_link board_l152 =                /* Board Nucleo L152                  */
  BEGIN                                /* ---------------------------------- */
    "L152"                          ,  /* Nom du board                       */
    drv_l152                        ,  /* Liste des drivers                  */
  END                                ; /* ---------------------------------- */


Board_link *boards[] =                 /* Definitions des boards             */
 BEGIN                                 /* ------------ |MC  |CPU| RAM|Flash| */
  &board_l152                       ,  /* 0 STM32L152  |L152|  0| 80K| 512K| */
 END                                 ; /* ---------------------------------+ */


Drv_itf_ctx drv_itf_ctx[] =
  BEGIN
    {asy_itfctx   ,NUM_ASY   ,1,{0}} ,
    {led_itfctx   ,LEDCTRL_NB,1,{0}} ,
    {gpio_itfctx  ,GPIO_NBCTL,1,{0}} ,
    {kbditf_itfctx,1         ,1,{0}} ,
  END                                ;

/*
    11.3 Tables de debug
    ____________________

         Afin d'aider a l'identification de bugs qui pourraient se produire
apres un temps de fonctionnement assez long, un certain nombre de tables de
debug sont renseignees par les differents modules du VMIO. Ces tables peuvent
avoir des tailles tres consequentes et il y a des cas ou on souhaite les
reduire au minimum (loader) et d'autre ou veut les augmenter au maximum (debug
d'un pb sur une platteforme donnee). On definit donc ces tables dans les
differents fichiers boards.

*/

/* Driver interruption "int_l152.c" ........................................ */

const IWORD max_note_it =   50     ; /* Taille de la table de debug          */
Note_it     tnote_it     [  50]    ; /* Buffer circulaire de notes           */


/* Driver dma "dma_l152.c" ................................................. */

const IWORD max_note_dma = 10      ; /* Taille de la table de debug          */
Note_dma    tnote_dma     [10]     ; /* Buffer circulaire de notes           */


/* VMIO "vmio.c" ........................................................... */

const IWORD max_trace_evt_s = 10   ; /* Taille de la table de debug          */
Err_evt_s   terr_evt_s       [10]  ; /* Buffer circulaire de notes           */

const IWORD max_note_evt_s =  20   ; /* Taille de la table de debug          */
Note_evt_s  tnote_evt_s      [20]  ; /* Buffer circulaire de notes           */



/*
    12   Procedures des drivers systemes/caracteres du board
    ________________________________________________________
*/

void set_func_stm32(int,int,int) ;
void set_pull_stm32(int,int,int) ;


/*  Procedure board_init --------------------------------------------------- */
/*
    But : Initialiser les parametres globaux du board.
 
    Auteur : AL                              Derniere modification : 13/11/2017

...............................................................................
 
    Exemple d'appel : board_init()
 
    En entree : -
 
    En sortie : -
 
    Notes : Cette procedure est specifique au board.

            NB : Le driver "board" est le 2eme dans l'ordre d'initialisation
                 des drivers systeme, apres le driver "sys" qui initialise
                 les pointeurs sur les registres des devices.
 
...............................................................................
*/
static IWORD PROCEDURE board_init()

  BEGIN                                /* Debut de cette procedure.          */

/*  Declaration des variables locales, internes de cette procedure ..........*/

/*  Debut du code ...........................................................*/

#ifdef debug                           /* Debut compilation conditionnelle.  */
    procdeb("board_init")            ; /* Message.                           */
#endif                                 /* Fin compilation conditionnelle.    */

/*****************************************************************************
 * Etape 1 : On active l'usage des mises-en-sommeil du processeur par des    *
 * -------   appels a "vm_sleep".                                            *
 *****************************************************************************/

//  set_idle(1)                      ; /* On active la fonction.             */


/*****************************************************************************
 * Etape 2 : Si necessaire pour debugguer le GPIO, on met les pins JTAG dans *
 * -------   le mode adapte. Sinon, on est un peu coince.                    *
 *****************************************************************************/

//  set_func_stm32(0,13,1      )     ; /* PA13 JTMS-SWDIO  JTAG              */
//  set_pull_stm32(0,13,GPIO_PU)     ; /*      Pull-up                       */
//  set_func_stm32(0,14,1      )     ; /* PA14 JTCK-SWCLK  JTAG              */
//  set_pull_stm32(0,14,GPIO_PD)     ; /*      Pull-down                     */
//  set_func_stm32(0,15,1      )     ; /* PA15 RJTDI       JTAG              */
//  set_pull_stm32(0,15,GPIO_PU)     ; /*      Pull-up                       */


#ifdef debug                           /* Debut compilation conditionnelle.  */
    procfin("board_init")            ; /* Message.                           */
#endif                                 /* Fin compilation conditionnelle.    */

    return (0)                       ; /* Code de retour de la procedure.    */

  END_PROC




/*  Procedure vm_sleep ------------------------------------------------------*/
/*
    But : Ne rien faire quand on nous propose de dormir.

    Auteur : PJB                             Derniere modification : 15/10/2015

...............................................................................

    Exemple d'appel : vm_sleep(par)

    En entree : par      IWORD      Parametre : 0 pour ignore, 1 pour dormir.

    En sortie : procval  IWORD      0.

    Notes : On ne fait jamais rien ici.

...............................................................................
*/
IWORD PROCEDURE vm_sleep(IWORD par)

  BEGIN                                /* Debut de cette procedure           */

/* Declaration des variables locales, internes a cette procedure ............*/

/* Debut du code ............................................................*/

#ifdef debug                           /* Debut compilation conditionnelle   */
    procdeb("vm_sleep")              ; /* Message                            */
#endif                                 /* Fin compilation conditionnelle     */

    if (par) asm("WFI")              ; /* Si par = 1, on utilise WFI.        */

#ifdef debug                           /* Debut compilation conditionnelle   */
    procfin("vm_sleep")              ; /* Message                            */
#endif                                 /* Fin compilation conditionnelle     */

    return 0                         ; /* Code de retour de la procedure     */

  END_PROC                             /* Fin de la procedure                */



/*  Procedure infoLED -------------------------------------------------------*/
/*
    But : Remonter les informations du controleur.

    Auteur : AL                              Derniere modification : 22/11/2017

...............................................................................

    Exemple d'appel : infoLED(dev,&name,&nbleds,&colors,&states)

    En entree : dev      IWORD      Numero du device (controleur de leds)

    En sortie : name     const CHAR**  Adresse de  la chaine contenant le nom
                                         du circuit
                nbleds   IWORD*        Nombre de leds raccordees au device
                colors   const UCHAR** Tableau (de taille "nbleds") donnant la
                                         couleur de chaque led raccordee au
                                         device
                states   const UCHAR** Tableau (de taille "nbleds") donnant
                                         l'etat initial de chaque led raccordee
                                         au device


    Notes : Cette procedure est specifique au board.

...............................................................................
*/
static void PROCEDURE infoLED(IWORD dev, const CHAR **name, IWORD *nbleds,
                              const CHAR ***names, const UCHAR **colors,
                              const UCHAR **states                         )
                               

  BEGIN                                /* Debut de cette procedure           */

/* Declaration des variables locales, internes a cette procedure ............*/

/* Debut du code ............................................................*/

#ifdef debug                           /* Debut compilation conditionnelle   */
    procdeb("infoLED")               ; /* Message                            */
#endif                                 /* Fin compilation conditionnelle     */

    *name   = "Nucleo L152RE"        ; /* Nom du board.                      */
    *nbleds = LED_NB                 ; /* Nombre de LEDs pilotables.         */
    *names  = lednames               ; /* Table des couleurs.                */
    *colors = ledcol                 ; /* Table des couleurs.                */
    *states = ledsta                 ; /* Table des etats.                   */

#ifdef debug                           /* Debut compilation conditionnelle   */
    procfin("infoLED")               ; /* Message                            */
#endif                                 /* Fin compilation conditionnelle     */

    return                           ; /* Code de retour de la procedure     */

  END_PROC                             /* Fin de la procedure                */




/*  Procedure setLED --------------------------------------------------------*/
/*
    But : Modifier l'etat d'une LED.

    Auteur : AL                              Derniere modification : 22/11/2017

...............................................................................

    Exemple d'appel : setLED(dev, ssc, etat)

    En entree : dev      IWORD      Numero du device (controleur de leds)
                sscanal  IWORD      Numero du sous-canal (led)
                etat     IWORD      Allume (1) ou eteint (0)

    En sortie : -

    Notes : Cette procedure est specifique au board.

...............................................................................
*/
static void PROCEDURE setLED(IWORD dev, IWORD ssc, IWORD etat)

  BEGIN                                /* Debut de cette procedure           */

/* Declaration des variables locales, internes a cette procedure ............*/

/* Debut du code ............................................................*/

#ifdef debug                           /* Debut compilation conditionnelle   */
    procdeb("setLED")                ; /* Message                            */
#endif                                 /* Fin compilation conditionnelle     */

    set_pio(ledpio[ssc][0],ledpio[ssc][1],etat) ;

#ifdef debug                           /* Debut compilation conditionnelle   */
    procfin("setLED")                ; /* Message                            */
#endif                                 /* Fin compilation conditionnelle     */

    return                           ; /* Code de retour de la procedure     */

  END_PROC                             /* Fin de la procedure                */






#if 0

/*  Procedure vm_sleep ------------------------------------------------------*/
/*
    But : Realiser une mise en veille.

    Auteur : PJB                             Derniere modification : 07/10/2015

...............................................................................

    Exemple d'appel : vm_sleep(val)

    En entree : val      IWORD      Parametre de la veille :
                                    - SLEEP_NONE  (0), pas de veille
                                    - SLEEP_CPU   (1), arret de l'horloge CPU
                                    - SLEEP_MOST  (2), arret de presque tout
                                    - SLEEP_SHTDWN(3), arret complet

    En sortie : procval  IWORD      0 si OK, -1 sinon

    Notes :

...............................................................................
*/
IWORD vm_sleep(IWORD) __attribute__((section(".text.ram")));
IWORD PROCEDURE vm_sleep(IWORD val)

  BEGIN                                /* Debut de cette procedure           */

/* Declaration des variables locales, internes a cette procedure ............*/

    IWORD          ret = -1          ; /* Code de retour de la procedure.    */

extern volatile Rcc_ctrl* const rcc  ; /*                                    */
extern volatile Scs     * const scs  ; /*                                    */
extern volatile UINT    * const pwr  ; /*                                    */

#define SLEEP_NONE 0
#define SLEEP_CPU  1
#define SLEEP_MOST 2

/* Debut du code ............................................................*/

#ifdef debug                           /* Debut compilation conditionnelle   */
    procdeb("vm_sleep")              ; /* Message                            */
#endif                                 /* Fin compilation conditionnelle     */

    switch (val)                       /* On saute a l'etape selon la valeur */
                                       /* de "val".                          */
      BEGIN                            /* ---------------------------------- */

/*****************************************************************************
 * Etape 0 : Aucune veille                                                   *
 * -------   Si val = SLEEP_NONE (=0) ou que c'est une valeur farfelue, on   *
 *           ne fait rien.                                                   *
 *****************************************************************************/
    default : vhalt()                ; /* Valeur inattendue : on vient voir  */
    ret = -1                         ; /*                                    */

    case SLEEP_NONE  :                 /*                                    */
    goto exit                        ; /*                                    */


/*****************************************************************************
 * Etape 1 : Veille du CPU seul                                              *
 * -------   Si val = SLEEP_CPU (=1), on met le CPU en attente. Cela         *
 *           correspond au mode sleep du STM32L1xx. Pour entrer dans ce      *
 *           mode, il suffit d'attendre une interruption : WFI               *
 *           Si on n'a pas desactive le compteur de milisecondes, on va etre *
 *           reveille bientot.                                               *
 *           Pour rappel : STM32L152RE @ 2Mhz se reveille en 7us vs 1ms au   *
 *           plus.                                                           *
 *           Attention : le mode low-power sleep peut sembler interessant    *
 *           (consommation 4 fois moindre), mais le temps de reveil est bien *
 *           plus grand. Il vaut mieux faire l'effort de trouver les vrais   *
 *           temps de veille et passer en SLEEP_MOST.                        *
 *****************************************************************************/
    case SLEEP_CPU   :                 /*                                    */

    scs->sc = 0                      ; /* Not about to deepsleep             */
    asm("WFI")                       ; /* On se met en attente d'une IT.     */

    goto exit                        ; /*                                    */

/*****************************************************************************
 * Etape 2 : Veille du CPU et des peripheriques (sauf le minimum pour etre   *
 * -------   reveille quand meme). On garde aussi la RAM.                    *
 *           Pour le STM32L1xx :                                             *
 *           - On s'assure que la RTC est bien active, sinon bonne-nuit!     *
 *           - On s'assure que l'oscillateur lent externe est bien active.   *
 *           - On leve le bit sleepdeep.                                     *
 *           - On baisse le bit WUP de PWR_CR. Ce bit indique qu'un reveil   *
 *             a eu lieu, et il empeche de passer en veille profonde.        *
 *           - On leve les bits FWU et ULP. Le second permet de desactiver   *
 *             le maintien de Vrefint pendant que l'on passe en mode basse-  *
 *             consommation. Le premier permet de sortir de basse consom-    *
 *             mation avant que Vrefint ne soit revenu a sa valeur.          *
 *           - On active le mode low power du regulateur : cela devrait      *
 *             reduire la consommation (mais on n'a pas la valeur de cette   *
 *             reduction), et augmenter le temps de remise en veille. Ce     *
 *             temps passe de 8 a 10us en moyenne ...                        *
 *           - Enfin, on utilise l'instruction WFI.                          *
 *****************************************************************************/
    case SLEEP_MOST  :                 /*                                    */

    if ((rcc->csr & 0x430300) NE       /* On verifie RTC, et LSE             */
                    0x410300   )       /*                                    */
      BEGIN                            /* En cas d'erreur, on vient voir,    */
        vhalt()                      ; /* mais on ne se met pas en veille.   */
        goto exit                    ; /* On sort a la place.                */
      END_IF                           /*                                    */

    scs->sc = 4                      ; /* About to deepsleep                 */
    pwr[0] &= ~0x602                 ; /* Clear FWU, ULP, PDDS               */
    pwr[0] |=      1                 ; /* Enables low power deepsleep        */

    asm("WFI")                       ; /* On se met en attente d'une IT.     */

    goto exit                        ; /*                                    */


    END                                /* ---------------------------------- */


    exit :                             /*                                    */

#ifdef debug                           /* Debut compilation conditionnelle   */
    procfin("vm_sleep")              ; /* Message                            */
#endif                                 /* Fin compilation conditionnelle     */

    return ret                       ; /* Code de retour de la procedure     */

  END_PROC                             /* Fin de la procedure                */
#endif
