/*****************************************************************************/
/*                                                                           */
/*  HyperPanel VMIO                     File "vmio/sys/vmiodat_iotc.c"       */
/*  VMIO static tables.                                                      */
/*                                                                           */
/*  Author : PJB/SBH                    Last modification : 17/11/2015       */
/*                                                                           */
/*  (c) Copyright HyperPanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================



    1    Purpose of this module
    ___________________________


         This module defines the static tables of the VMIO.



    2    User manual
    ________________


         A module like the VMIO can be implemented on platforms of different
sizes. We want to be able to use it without having to recompile the module for
each use. We have therefore extracted from the VMIO the tables in RAM as well
as the 4 constants that sized them in a module apart from depends on the
intended use of the VMIO. So we define both the static tables of the VMIO and
the constant variables that allow the VMIO to manipulate them.

         The dimensioning of the VMIO is done by adjusting the constants
below:

#CODE -------------------------------------------------------------------------
          MAXAUT_S 
          MAXEVT_S
          MAXRTE_S 
          NTOMAX_S.
#END --------------------------------------------------------------------------


         Note that the constants MAXDUR_S and MAXDUR_IT are still defined
elsewhere (in "vmiodat.h") and used elsewhere (in "vmio.c", "int_xyz.c") and
still require a recompilation of these modules if we modify them. For MAXDUR_S
is unavoidable because it is the second dimension of a table with 2 dimensions,
which must be known in all modules that use it. For MAXDUR_IT we could do
without it if we wanted, as we did for the other definitions but it's not done.



    3    Porting guide
    __________________



    4    Include files and external references
    __________________________________________
*/


#include <stdtyp.l>                    /* HyperPanel definitions & constants.*/
#include <moteur.h>                    /* S_evt.                             */
#include <vmiodat.h>                   /* Fah_s, MAXDUR_S.                   */
#include <vmiodbg.h>                   /* Dura_fa_s,Dura_trt_s, etc ...      */
                                       /*                                    */
#ifdef XDRV                            /* New driver interface.              */
#include <xauto.h>                     /* AUTS_FIRST                         */
#else                                  /* Previous driver interface.         */
#include <auto.h>                      /* AUTS_FIRST                         */
#endif                                 /* End of ifdef XDRV.                 */


/* From the linker ......................................................... */

extern IWORD     itstkl              ; /* Ad. basse des piles d'IT           */
extern IWORD     itstkh              ; /* Ad. haute des piles d'IT           */


/*
    5    Description of the internal behaviour of the module
    ________________________________________________________
*/


/*
    6    Exported (public) variables
    ________________________________
*/


/* Le redimensionnement du VMIO devrait s'en tenir a ces 4 constantes .......*/

#define MAXAUT_S             16        /* Nombre d'automate maxi.            */
#define MAXEVT_S             16        /* Taille de la file d'attente        */
#define NTOMAX_S              8        /* Nombre maximum de temporisateurs   */
#define MAXRTE_S              4        /* Nombre maximum de routes           */


/* Tout le reste du module devrait etre le meme partout .....................*/

#define AUTS_LAST  (AUTS_FIRST+MAXAUT_S-1)

const INT      maxevt_s  = MAXEVT_S  ; /* Nombre d'evenements ("fa_s")       */
const INT      maxaut_s  = MAXAUT_S  ; /* Nombre d'automates                 */
const INT      auts_last = AUTS_LAST ; /* Indice du dernier automate         */


/* Les trois files d'attente ................................................*/

#define FAL                   0        /*                                    */
#define FAN                   1        /*                                    */
#define FAU                   2        /*                                    */
#define FFREE                 3        /*                                    */

const INT      fal_idx = FAL         ; /* Indice de la FAL   = maxaut_s + 0  */
const INT      fan_idx = FAN         ; /* Indice de la FAN   = maxaut_s + 1  */
const INT      fau_idx = FAU         ; /* Indice de la FAU   = maxaut_s + 2  */
const INT      faf_idx = FFREE       ; /* Indice de la FFREE = maxaut_s + 3  */
S_evt          fa_s  [MAXEVT_S]      ; /* Elements des files d'attentes      */
Fah_s          fah_s [       4]      ; /* Les tetes de chaines "files"       */


/* Temporisateurs ...........................................................*/

const INT      ntomax_s  = NTOMAX_S  ; /* Nombre de temporisateurs           */
S_to_s         tabto_s[NTOMAX_S]     ; /* Tableau de structures TO           */


/* Routes ...................................................................*/

const INT      maxrte_s  = MAXRTE_S  ; /* Nombre de temporisateurs           */
Route          route_s[MAXRTE_S]     ; /* Table de routage.                  */


/* Moteur d'automates .......................................................*/

USHORT         *tablim_s  [MAXAUT_S] ; /* Tableaux des limites d'etat        */
USHORT         *varetat_s [MAXAUT_S] ; /* Tableaux des variables d'etat      */
S_trans        *tabtrans_s[MAXAUT_S] ; /* Tables de transition               */


/* Adresses des piles sous interruptions ................................... */

#define SZ0                   0        /* Taille pile IT priorite 0 (octets) */
#define SZ1              2*1024        /* Taille pile IT priorite 1 (octets) */
#define SZ2                 512        /* Taille pile IT priorite 2 (octets) */
#define SZ3                 512        /* Taille pile IT priorite 3 (octets) */
#define SZ4                 512        /* Taille pile IT priorite 4 (octets) */
#define SZ5                 512        /* Taille pile IT priorite 5 (octets) */
#define SZ6                 512        /* Taille pile IT priorite 6 (octets) */
#define SZ7                   0        /* Taille pile IT priorite 7 (octets) */

#define OF0          (      SZ0)       /* Offset pour la base de la pile 0   */
#define OF1          (OF0 + SZ1)       /* Offset pour la base de la pile 1   */
#define OF2          (OF1 + SZ2)       /* Offset pour la base de la pile 2   */
#define OF3          (OF2 + SZ3)       /* Offset pour la base de la pile 3   */
#define OF4          (OF3 + SZ4)       /* Offset pour la base de la pile 4   */
#define OF5          (OF4 + SZ5)       /* Offset pour la base de la pile 5   */
#define OF6          (OF5 + SZ6)       /* Offset pour la base de la pile 6   */
#define OF7          (OF6 + SZ7)       /* Offset pour la base de la pile 7   */

const IWORD *itstk[8] =                /* Table des adresses des piles d'IT  */
  BEGIN                                /* ---------------------------------- */
    &itstkl + OF0/ZW                 , /* Pile pour IT de priorite 0         */
    &itstkl + OF1/ZW                 , /* Pile pour IT de priorite 1         */
    &itstkl + OF2/ZW                 , /* Pile pour IT de priorite 2         */
    &itstkl + OF3/ZW                 , /* Pile pour IT de priorite 3         */
    &itstkl + OF4/ZW                 , /* Pile pour IT de priorite 4         */
    &itstkl + OF5/ZW                 , /* Pile pour IT de priorite 5         */
    &itstkl + OF6/ZW                 , /* Pile pour IT de priorite 6         */
    &itstkl + OF7/ZW                 , /* Pile pour IT de priorite 7         */
  END                                ; /* ---------------------------------- */

const IWORD *endstk[8] =               /* Table des adresses fin de piles    */
  BEGIN                                /* ---------------------------------- */
    &itstkl + OF0/ZW - SZ0/ZW        , /* Pile pour IT de priorite 0         */
    &itstkl + OF1/ZW - SZ1/ZW        , /* Pile pour IT de priorite 1         */
    &itstkl + OF2/ZW - SZ2/ZW        , /* Pile pour IT de priorite 2         */
    &itstkl + OF3/ZW - SZ3/ZW        , /* Pile pour IT de priorite 3         */
    &itstkl + OF4/ZW - SZ4/ZW        , /* Pile pour IT de priorite 4         */
    &itstkl + OF5/ZW - SZ5/ZW        , /* Pile pour IT de priorite 5         */
    &itstkl + OF6/ZW - SZ6/ZW        , /* Pile pour IT de priorite 6         */
    &itstkl + OF7/ZW - SZ7/ZW        , /* Pile pour IT de priorite 7         */
  END                                ; /* ---------------------------------- */


/* Mesures des durees les plus grandes en file ..............................*/

Dura_fa_s      tdura_fa_s[3][MAXDUR_S]                  ;
const UINT     tdura_fa_s_sze = sizeof(tdura_fa_s)      ;


/* Mesures des durees de traitement les plus longues ........................*/

Dura_trt_s     tdura_trt_s[MAXAUT_S][MAXDUR_S]          ;
const UINT     tdura_trt_s_sze = sizeof(tdura_trt_s)    ;


/* Mesures des durees de handler les plus longues ...........................*/

Dura_hand_it   tdura_hand_it[MAXDUR_IT]                 ;
const UINT     tdura_hand_it_sze = sizeof(tdura_hand_it);


/* Mesures des durees de section critique les plus longues ..................*/

Dura_crit_it   tdura_crit_it[MAXDUR_IT]                 ;
const UINT     tdura_crit_it_sze = sizeof(tdura_crit_it);


/*
    7    Internal constants of the module
    _____________________________________
*/


/*
    8    Internal data types of the module
    ______________________________________
*/


/*
    9    Internal (private) variables of the module
    _______________________________________________
*/


/*
    10   Beginning of the code
    __________________________
*/

/* Following procedures shall not be linked with this module ................*/

PROCEDURE IWORD stoevt_s(S_evt *a)
      { return -1 ; }

PROCEDURE IWORD getevt_s(IWORD a, IWORD *b, IWORD c, IWORD d, S_evt *e)
      { return 0 ; }

PROCEDURE IWORD clrevt_s(IWORD a)
      { return 0 ; }
